@@ -7,7 +7,7 @@ use diagnostics;
 require Exporter;
 our @ISA = qw(Exporter);
 our @EXPORT = qw( auction );
-our $VERSION = '0.23';
+our $VERSION = '0.25';
 
 #Variables global to the package	
 my $maximize_total_benefit;
@@ -25,13 +25,13 @@ my %index_correlation;
 sub auction { #                        => default values
    my %args = ( matrix_ref             => undef,     # reference to array: matrix N x M			                                     
                 maximize_total_benefit => 0,         # 0: minimize_total_benefit ; 1: maximize_total_benefit
-				stepsize               => 1,
+				inicial_stepsize       => undef,     # auction algorithm terminates with a feasible assignment if the problem data are integer and stepsize < 1/min(N,M)				
 				inicial_price          => 0,
 				verbose                => 3,         # level of verbosity, 0: quiet; 1, 2, 3, 4, 9, 10: debug information.				
                 @_,                                  # argument pair list goes here
 	          );
        
-   my @matrix_input = @{$args{matrix_ref}};          # Input: Reference to the input matrix (NxM)  
+   my @matrix_input = @{$args{matrix_ref}};          # Input: Reference to the input matrix (NxM) = $min_size x $max_size
    
    $array1_size = $#matrix_input + 1;
    $array2_size = $#{$matrix_input[0]} + 1;
@@ -68,13 +68,14 @@ sub auction { #                        => default values
    
    get_matrix_info( \@matrix, $args{verbose} );
    
-   delete_multiple_columns( \@matrix, $min_size, $args{verbose} ) if ( $min_size >= 2 );
+   delete_multiple_columns( \@matrix, $args{verbose} ) if ( $min_size >= 2 and $min_size != $max_size );
      
    # epsilon is the stepsize and auction algorithm terminates with a feasible assignment if the problem data are integer and epsilon < 1/min(N,M).
    # There is a trade-off between runtime and the chosen stepsize. Using the largest possible increment accelerates the algorithm.
    
    $inicial_price = $args{inicial_price};
-   my $epsilon = $args{stepsize};
+   my $epsilon = 1;
+	  $epsilon = $args{inicial_stepsize} if ( defined $args{inicial_stepsize} );
    my $feasible_assignment_condition = 0;
 
    # The preceding observations suggest the idea of epsilon-scaling, which consists of applying the algorithm several times, 
@@ -161,8 +162,7 @@ sub transpose {
 }
 
 sub delete_multiple_columns { # if the column elements do not change the final result
-   my ( $matrix_ref, $N, $verbose ) = @_;
-   my @matrix = @$matrix_ref;
+   my ( $matrix_ref, $verbose ) = @_;
    my %lower_values;
    my %intersection_columns;
    my $number_of_columns_deleted = 0;	  
@@ -172,13 +172,16 @@ sub delete_multiple_columns { # if the column elements do not change the final r
 	     $lower_values{ $i }{ $matrix_ref->[$i]->[$j] }{ $j }++;
       }
    } 
-
+   
+   # consider N rows < M columns
+   # remove the matching columns whose elements are never among the N largest elements in each row
+   
    foreach my $index_i ( sort { $a <=> $b } keys %lower_values ){
       my $num_higher_values = 0;
       foreach my $matrix_value ( sort { $b <=> $a } keys %{$lower_values{$index_i}} ){
 	     foreach my $index_j ( sort { $b <=> $a } keys %{$lower_values{$index_i}{$matrix_value}} ){	     
-		    $intersection_columns{$index_j}++ if ( ++$num_higher_values > $N );
-			$number_of_columns_deleted++ if ( defined $intersection_columns{$index_j} and $intersection_columns{$index_j} >= $N );
+		    $intersection_columns{$index_j}++ if ( $num_higher_values++ >= $min_size );
+			$number_of_columns_deleted++ if ( defined $intersection_columns{$index_j} and $intersection_columns{$index_j} >= $min_size );
 		 }
 	  }
    }
@@ -189,7 +192,7 @@ sub delete_multiple_columns { # if the column elements do not change the final r
          print " [";
          for my $j ( 0 .. $#{$matrix_ref->[$i]} ) {
             printf(" %2.0f", $matrix_ref->[$i]->[$j] );
-			if ( defined $intersection_columns{$j} and $intersection_columns{$j} == $N ){ print "**"; } else{ print "  "; }
+			if ( defined $intersection_columns{$j} and $intersection_columns{$j} == $min_size ){ print "**"; } else{ print "  "; }
          }
          print "]\n";
       }
@@ -199,13 +202,13 @@ sub delete_multiple_columns { # if the column elements do not change the final r
    my $idx = 0;  
    for my $i ( 0 .. $#{$matrix_ref} ) {
       for my $j ( 0 .. $#{$matrix_ref->[$i]} ) {
-	     undef ( $matrix_ref->[$i]->[$j] ) if ( defined $intersection_columns{$j} and $intersection_columns{$j} >= $N );
-		 if ( $i == 0 and defined $intersection_columns{$j} and $intersection_columns{$j} >= $N ){
-		    print " \$N = $N ; \$intersection_columns{$j} = $intersection_columns{$j} \n" if ( $verbose >= 7 );
+	     undef ( $matrix_ref->[$i]->[$j] ) if ( defined $intersection_columns{$j} and $intersection_columns{$j} >= $min_size );
+		 if ( $i == 0 and defined $intersection_columns{$j} and $intersection_columns{$j} >= $min_size ){
+		    printf " N = $min_size ; M = $max_size ; j = %2s ; \$intersection_columns{$j} = $intersection_columns{$j} \n", $j if ( $verbose >= 7 );
 		 }
-		 if ( $i == 0 and ( not defined $intersection_columns{$j} or $intersection_columns{$j} < $N ) ){
+		 if ( $i == 0 and ( not defined $intersection_columns{$j} or $intersection_columns{$j} < $min_size ) ){
 		    $index_correlation{$idx} = $j;
-		    print " \$N = $N ; \$index_correlation{$idx} = $index_correlation{$idx} \n" if ( $verbose >= 7 );
+		    printf " N = $min_size ; M = $max_size ; j = %2s ; \$index_correlation{$idx} = $index_correlation{$idx} \n", $j if ( $verbose >= 7 );
 			$idx++;
 		 }
       }
@@ -332,7 +335,6 @@ sub print_screen_messages {
    }
 
 }
-   
 
 sub get_matrix_info {
    my ( $matrix_ref, $verbose ) = @_;
@@ -352,7 +354,7 @@ sub get_matrix_info {
    $decimals = length(($max_matrix_value =~ /[,.](\d+)/)[0]); # counting the number of digits after the decimal point
    $decimals = 0 unless ( defined $decimals );                # for integers $decimals = 0
    
-   my $range = $max_matrix_value - $min_matrix_value;           # $range >= 0
+   my $range = $max_matrix_value - $min_matrix_value;         # $range >= 0
       $range = 1 if ($range == 0);
    
    if ( $verbose >= 6 ){
@@ -364,9 +366,10 @@ sub get_matrix_info {
       for my $i ( 0 .. $#matrix ) {
       for my $j ( 0 .. $#{$matrix[$i]} ) {
 	     
-		 #$matrix[$i]->[$j] = $matrix[$i]->[$j] - $min_matrix_value ;
+		 $matrix[$i]->[$j] = $matrix[$i]->[$j] - $min_matrix_value ;
 		 
-	     $matrix[$i]->[$j] = 99 * ( $matrix[$i]->[$j] - $min_matrix_value ) / $range; # new scale: Min = 0 <---> Max = 99
+	     #$matrix[$i]->[$j] = 99 * ( $matrix[$i]->[$j] - $min_matrix_value ) / $range; # new scale: Min = 0 <---> Max = 99
+
       }}   
  	  
    } else {
@@ -374,9 +377,9 @@ sub get_matrix_info {
       for my $i ( 0 .. $#matrix ) {
       for my $j ( 0 .. $#{$matrix[$i]} ) {   
 	     
-		 #$matrix[$i]->[$j] = $max_matrix_value - $matrix[$i]->[$j] ;
+		 $matrix[$i]->[$j] = $max_matrix_value - $matrix[$i]->[$j] ;
 		 
-		 $matrix[$i]->[$j] = 99 * ( $max_matrix_value - $matrix[$i]->[$j] ) / $range;
+		 #$matrix[$i]->[$j] = 99 * ( $max_matrix_value - $matrix[$i]->[$j] ) / $range;
       }}	  
    }
 }
@@ -387,7 +390,7 @@ sub auctionRound {
 	my %info;
 
    if ( $verbose >= 10 ){
-      print "\n Start: Matrix Size N x M: $max_size x $max_size ; Num Iterations: $iter_count ; epsilon: $epsilon \n";
+      print "\n Start: Matrix Size N x M: $min_size x $max_size ; Num Iterations: $iter_count ; epsilon: $epsilon \n";
 	  foreach my $person ( sort { $a <=> $b } keys %assignned_person ){
 	     my $object = $assignned_person{$person};
 	     printf " \$assignned_person{%2s} --> object %2s --> \$price_object{%2s} = $price_object{$object} \n", $person, $object, $object;
@@ -402,7 +405,7 @@ sub auctionRound {
 
 	for my $person ( 0 .. $max_size - 1 )
 	{		
-		last if $seen_ghost; # don't need to fill the matrix with zeros, that is, don't need to convert rectangular N x M to square matrix by padding zeroes.
+		last if $seen_ghost; # don't need to fill the matrix with zeros, that is, don't need to convert rectangular N x M to square matrix by padding zeroes. Need just one more row: N+1 x M
 		
 		if ( not defined $assignned_person{$person} )
 		{		
@@ -426,21 +429,21 @@ sub auctionRound {
 				$price_object{$object} = $inicial_price unless ( defined $price_object{$object} );				
 				my $curVal = $matrix_value - $price_object{$object};				
 				
-				if ( (not defined $optValForI) || ($curVal > $optValForI) )
+				if ( (not defined $optValForI) || ($curVal > $optValForI) )          # best object value
 				{
 					$secOptValForI = $optValForI;
 					$secOptObjForI = $optObjForI;
 					$optValForI = $curVal;
 					$optObjForI = $object;
 				}
-				elsif ( (not defined $secOptValForI) || ($curVal > $secOptValForI) )
+				elsif ( (not defined $secOptValForI) || ($curVal > $secOptValForI) ) # second best object value
 				{
 					$secOptValForI = $curVal;
 					$secOptObjForI = $object;
 				}
 				
 				if ( $verbose >= 10 ){
-			        printf " personI = %2s ; objectJ = %2s ; N = %2s ; M = %2s ; \$curVal %6.2f = \$matrix[%2s]->[%2s] %2.0f - \$price_object{%2s} %6.2f ;", $person, $object, $array1_size, $array2_size, $curVal, $person, $object, $matrix_value, $object, $price_object{$object};				
+			        printf " personI = %2s ; objectJ = %2s ; N = %2s ; M = %2s ; \$curVal %6.2f = \$matrix[%2s]->[%2s] %2.0f - \$price_object{%2s} %6.2f ;", $person, $object, $min_size, $max_size, $curVal, $person, $object, $matrix_value, $object, $price_object{$object};				
 				    if ( defined $optValForI    ){ printf " \$optValForI = %6.2f ", $optValForI; }
 				    if ( defined $secOptValForI ){ printf " \$secOptValForI = %6.2f \n", $secOptValForI; } else { print "\n"; }
                 }
@@ -449,33 +452,36 @@ sub auctionRound {
 			## Computes the highest reasonable bid for the best object for this person
 			my $bidForI = $optValForI - $secOptValForI + $epsilon;			
 
-			## Stores the bidding info for future use			
-			$info{$optObjForI}{$bidForI}{$person}++ if ( not defined $info{$optObjForI}{$bidForI} ); # first person with this bid
+            # Stores the bidding info for future use			
+			$info{$optObjForI}{$bidForI} = $person if ( not defined $info{$optObjForI}{$bidForI} ); # get only the first person with this bid
 			
-			if ( $verbose >= 10 ){
+			if ( $verbose >= 10 ){			    
 			    printf "<> PersonI = %2s ; ObjectJ = %2s ; \$bidForI %3s = \$optValForI %3s - \$secOptValForI %3s + \$epsilon %.4f \n", $person, $optObjForI, $bidForI, $optValForI, $secOptValForI, $epsilon;
 		    }
 		}
 	}
 	
-	my %seen_first_object;
+	LOOP1: foreach my $object ( sort { $a <=> $b } keys %info ){              # ascending order      for each object, choose only the first bid.
+	LOOP2: foreach my $bid    ( sort { $b <=> $a } keys %{$info{$object}} ){  # descending order!!!  the first bid is the highest bid.
 	
-	foreach my $object ( sort { $a <=> $b } keys %info ){                      # ascending order
-	foreach my $bid    ( sort { $b <=> $a } keys %{$info{$object}} ){          # descending order!!!
-	foreach my $person ( sort { $a <=> $b } keys %{$info{$object}{$bid}} ){    # ascending order	   
-	   
-	   next if ( $seen_first_object{$object}++ );  # choose only the highest Bid For ObjectJ, 
-	   
-	   if ( defined $assignned_object{$object} ) { # Find the other person who has object $j and make them unassigned	      
-          
-		  my $other_p = $assignned_object{$object};
+	   my $person       = $info{$object}{$bid};
+	   my $other_person = $assignned_object{$object}; # Find the other person who has object $j and make them unassigned
+	   	   	   
+	   if ( defined $other_person ) {
 		  
 	      if ( $verbose >= 10 ){
-			 print " ***--> PersonI $person was assigned objectJ $object. Before that, remove the objectJ $object from personI $other_p  --> delete \$assignned_person{$other_p} \n";
+			 print " ***--> PersonI $person was assigned objectJ $object. Before that, remove the objectJ $object from personI $other_person  --> delete \$assignned_person{$other_person} \n";
 	      }
-		  delete $assignned_person{$other_p};
+		  
+		  # The other person that was assigned to objectJ at the beginning of the iteration (if any) 
+	      # is now left without an object (and becomes eligible to bid at the next iteration).
+	   
+		  delete $assignned_person{$other_person};
 	   }
 	   
+	   # Each objectJ that receives one or more bids, determines the highest of these bids, increases the price_j 
+	   # to the highest bid, and gets assigned to the personI who submitted the highest bid.
+	   
 	   $assignned_person{$person} = $object;
 	   $assignned_object{$object} = $person;
 	   
@@ -484,11 +490,13 @@ sub auctionRound {
 	   if ( $verbose >= 10 ){
 	      printf " --> Assigning to personI = $person the objectJ = $object with highestBidForJ = %6.2f and update the price vector ; \$assignned_person{$person} = $assignned_person{$person} ; \$price_object{$object} = $price_object{$object} \n", $bid;	
        }
-    }}}
+	   
+	   next LOOP1;  # next object, choose only the highest bid for each object
+    }}
 	
    # Prints the %assignned_person and %price_object
    if ( $verbose >= 10 ){
-      print "\n Final: Matrix Size N x M: $max_size x $max_size ; Num Iterations: $iter_count ; epsilon: $epsilon \n";
+      print "\n Final: Matrix Size N x M: $min_size x $max_size ; Num Iterations: $iter_count ; epsilon: $epsilon \n";
 	  foreach my $person ( sort { $a <=> $b } keys %assignned_person ){
 	     my $object = $assignned_person{$person};
 	     printf " \$assignned_person{%2s} --> object %2s --> \$price_object{%2s} = $price_object{$object} \n", $person, $object, $object;
@@ -519,7 +527,7 @@ __END__
  effectiveness is optimised (Maximised or Minimised)."
  
  "Each assignment problem has associated with a table or matrix. Generally, the rows contain the 
- jobs or people we wish to assign, and the columns comprise the tasks or objects we want them 
+ jobs (or persons) we wish to assign, and the columns comprise the tasks (or objects) we want them 
  assigned to. The numbers in the table are the costs associated with each particular assignment."
  
  One application is to find the (nearest/more distant) neighbors. 
@@ -534,10 +542,8 @@ __END__
 
  my @array1 = ( 19, 74, 58,  1 )
  my @array2 = ( 94, 55, 90, 45, 56, 95, 90 );
- 
- my $min = $#array1 < $#array2 ? $#array1 : $#array2;
- my $max = $#array1 < $#array2 ? $#array2 : $#array1;
 
+ my @input_matrix;
  for my $i ( 0 .. $#array1 ){
    my @weight_function;		   
    for my $j ( 0 .. $#array2 ){
@@ -546,37 +552,39 @@ __END__
       push @weight_function, $weight;
    }
    push @input_matrix, \@weight_function;
- } 
+ }
  
+ Alternatively, we can define the matrix with its elements:
+
+ my @input_matrix = (
+ [ 75, 36, 71, 26, 37, 76, 71 ],
+ [ 20, 19, 16, 29, 18, 21, 16 ],
+ [ 36,  3, 32, 13,  2, 37, 32 ],
+ [ 93, 54, 89, 44, 55, 94, 89 ]
+ );
+
       94 55 90 45 56 95 90
 
  19 [ 75 36 71 26 37 76 71 ]
  74 [ 20 19 16 29 18 21 16 ]
  58 [ 36  3 32 13  2 37 32 ]
   1 [ 93 54 89 44 55 94 89 ]
-
- @input_matrix = (
- [ 75, 36, 71, 26, 37, 76, 71 ],
- [ 20, 19, 16, 29, 18, 21, 16 ],
- [ 36,  3, 32, 13,  2, 37, 32 ],
- [ 93, 54, 89, 44, 55, 94, 89 ]
- );
  
  my ( $optimal, $assignement_ref, $output_index_ref ) = auction( matrix_ref => \@input_matrix, maximize_total_benefit => 1, verbose => 3 );
 
-Objective --> to Maximize the total benefit
+ Objective --> to Maximize the total benefit
  Number of left nodes: 4
  Number of right nodes: 7
  Number of edges: 28
 
-Solution
+ Solution
  Optimal assignment, sum of values = 230
  Feasible assignment condition: stepsize = 0.2 < 1/4 = 0.25
  Number of iterations: 18
 
-Maximum index size    = [ 0  1  2  3  4  5  6 ]
-@output_index indexes = [ 5  3  6  0  2  1  4 ]
-@output_index values  = [76 29 32 93          ]
+ Maximum index size    = [ 0  1  2  3  4  5  6 ]
+ @output_index indexes = [ 5  3  6  0  2  1  4 ]
+ @output_index values  = [76 29 32 93          ]
 
  original matrix 4 x 7 with solution:
  [ 75   36   71   26   37   76** 71  ]
@@ -593,7 +601,6 @@ Maximum index size    = [ 0  1  2  3  4  5  6 ]
    indexes ( 5, 1 ), weight value =    ; sum of values = 230
    indexes ( 6, 4 ), weight value =    ; sum of values = 230
 
-
  Common use of the solution:
    
  foreach my $array1_index ( sort { $a <=> $b } keys %{$assignement_ref} ){     
@@ -602,17 +609,18 @@ Maximum index size    = [ 0  1  2  3  4  5  6 ]
    ...
  }
  
- for my $i (0 .. $max){
+ for my $i ( 0 .. $#{$output_index_ref} ){
    my $j = $output_index_ref->[$i];
    ...
  }
-
  
 =head1 OPTIONS
  
- matrix_ref => \@input_matrix     reference to array: matrix N x M.
- maximize_total_benefit => 0      0: minimize the total benefit ; 1: maximize the total benefit.
- verbose  => 3                    print messages on the screen, level of verbosity, 0: quiet; 1, 2, 3, 4, 9, 10: debug information.
+ matrix_ref => \@input_matrix,   reference to array: matrix N x M.
+ maximize_total_benefit => 0,    0: minimize the total benefit ; 1: maximize the total benefit.
+ inicial_stepsize       => 1,    auction algorithm terminates with a feasible assignment if the problem data are integer and stepsize < 1/min(N,M).
+ inicial_price          => 0,			
+ verbose                => 3,    print messages on the screen, level of verbosity, 0: quiet; 1, 2, 3, 4, 9, 10: debug information.
 
 =head1 EXPORT
 
@@ -650,9 +658,9 @@ Maximum index size    = [ 0  1  2  3  4  5  6 ]
 	
 =head1 COPYRIGHT AND LICENSE
 
-Copyright (c) 2018 Claudio Fernandes de Souza Rodrigues.  All rights reserved.
+ Copyright (c) 2018 Claudio Fernandes de Souza Rodrigues.  All rights reserved.
 
-This program is free software; you can redistribute it and/or modify
-it under the same terms as Perl itself.
+ This program is free software; you can redistribute it and/or modify
+ it under the same terms as Perl itself.
 
 =cut
\ No newline at end of file
